require_relative 'player'
require_relative 'die'
require_relative 'game_turn'
require_relative 'treasure_trove'

module StudioGame
  class Game
    attr_reader :name
    def initialize(name)
      @name = name
      @players = []
    end

    def add_player(player)
      @players << player
    end

    def load_players(from_file)
      File.readlines(from_file).each do |line|
        add_player(Player.from_csv(line))
      end
    end

    def play(rounds)

      puts "There are #{@players.count} players in #{name}:"
      puts @players
      puts "There are #{TreasureTrove::TREASURES.size} treasures to be found."
      TreasureTrove::TREASURES.each { |treasure| puts "A #{treasure.name} is worth #{treasure.points} points"}
      1.upto(rounds) do |round_number|
        if block_given?
          if yield
            break
          end
        end
        puts "\nRound #{round_number}:"
        @players.each { |player|
          GameTurn.take_turn(player)
          puts player
        }
      end
    end

    def print_name_and_health(player)
      puts "#{player.name} (#{player.health})"
    end

    def print_stats
      puts "\n#{@name} High Scores:"
      @players.sort.each { |player| puts high_score_entry(player) }
      strongs, wimpys = @players.partition { |player| player.strong? }
      puts "\n#{@name} Statistics:"
      puts "\n#{strongs.size} strong players:"
      strongs.sort.each { |strong| print_name_and_health(strong) }
      puts "\n#{wimpys.size} wimpy players:"
      wimpys.sort.each { |wimpy| print_name_and_health(wimpy) }
      @players.sort.each { |player|
        puts "\n#{player.name}'s treasure point totals:"
        player.each_found_treasure { |treasure| puts "#{treasure.points} total #{treasure.name} points" }
        puts "#{player.points} grand total treasure points"
        }
      puts "\n#{total_points} total points from treasures found"
    end

    def total_points
      @players.reduce(0) { |total, player| total += player.points }
    end

    def high_score_entry(player)
      "#{player.name.ljust(20,".")} #{player.score}"
    end

    def save_high_scores
      to_file = File.join(File.dirname(File.dirname(File.dirname(__FILE__))), "high_scores.txt")
      File.open(to_file, "w") do |file|
        file.puts "\n#{@name} High Scores:"
          @players.sort.each { |player| file.puts high_score_entry(player)  }
      end
    end
  end
end

if __FILE__ == $0
  player1 = Player.new("moe")
  player2 = Player.new("larry", 60)

  two_knuckleheads = Game.new("Two Knuckleheads")
  two_knuckleheads.add_player(player1)
  two_knuckleheads.add_player(player2)
  two_knuckleheads.play(3)
  two_knuckleheads.print_stats
end
